@@ -8,7 +8,6 @@
 
 use std::{
     borrow::Cow,
-    ffi::CString,
     net::IpAddr,
     ops::Deref,
     os::{
@@ -27,6 +26,7 @@ use nix::{
 
 use crate::{
     compat::{addr_family, PF_ALG, PF_INET, PF_INET6, PF_MAX, PF_NETLINK, PF_UNIX, PF_UNSPEC},
+    config::HOOK_SCKCALLS,
     confine::{op2errno, op2name, scmp_arch_bits},
     fs::{fd_status_flags, file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
     hook::UNotifyEventRequest,
@@ -34,8 +34,10 @@ use crate::{
         accept::handle_accept,
         bind::handle_bind,
         connect::handle_connect,
+        getpeername::handle_getpeername,
         getsockname::handle_getsockname,
         getsockopt::handle_getsockopt,
+        recvfrom::handle_recvfrom,
         sendmsg::{handle_sendmmsg, handle_sendmsg},
         sendto::handle_sendto,
         socket::handle_socket,
@@ -48,8 +50,10 @@ use crate::{
 pub(crate) mod accept;
 pub(crate) mod bind;
 pub(crate) mod connect;
+pub(crate) mod getpeername;
 pub(crate) mod getsockname;
 pub(crate) mod getsockopt;
+pub(crate) mod recvfrom;
 pub(crate) mod sendmsg;
 pub(crate) mod sendto;
 pub(crate) mod socket;
@@ -59,26 +63,15 @@ const UNIX_PATH_MAX: usize = 108;
 pub(crate) fn sys_socketcall(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // Determine system call
-    // 0x1 => socket
-    // 0x2 => bind
-    // 0x3 => connect
-    // 0x5 => accept
-    // 0x6 => getsockname
-    // 0xb => sendto
-    // 0xf => getsockopt
-    // 0x10 => sendmsg
-    // 0x12 => accept4
-    // 0x14 => sendmmsg
-    #[expect(clippy::cast_possible_truncation)]
-    let op = match req.data.args[0] {
-        n @ (0x1 | 0x2 | 0x3 | 0x5 | 0x6 | 0xb | 0xf | 0x10 | 0x12 | 0x14) => n as u8,
-        _ => {
-            // SAFETY: Safe network call, continue.
-            // No pointer-dereference in access check.
-            return unsafe { request.continue_syscall() };
-        }
+    // Determine socket subcall.
+    let op: u8 = match req.data.args[0].try_into() {
+        Ok(op) => op,
+        Err(_) => return request.fail_syscall(Errno::EINVAL),
     };
+    if HOOK_SCKCALLS.binary_search(&op).is_err() {
+        // SAFETY: No pointer dereference in access check.
+        return unsafe { request.continue_syscall() };
+    }
 
     // Determine system call arguments.
     // On x86 unsigned long is 4 bytes, and on s390x 8 bytes.
@@ -142,6 +135,11 @@ pub(crate) fn sys_accept4(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x12)
 }
 
+pub(crate) fn sys_getpeername(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0x7)
+}
+
 pub(crate) fn sys_getsockname(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     syscall_network_handler(request, &req.data.args, 0x6)
@@ -157,6 +155,11 @@ pub(crate) fn sys_connect(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x3)
 }
 
+pub(crate) fn sys_recvfrom(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0xc)
+}
+
 pub(crate) fn sys_sendto(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     syscall_network_handler(request, &req.data.args, 0xb)
@@ -173,6 +176,7 @@ pub(crate) fn sys_sendmmsg(request: UNotifyEventRequest) -> ScmpNotifResp {
 }
 
 // A helper function to handle network-related syscalls.
+#[expect(clippy::cognitive_complexity)]
 fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let sandbox = request.get_sandbox();
@@ -192,15 +196,16 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
 
                 return handle_socket(&request, args, flags, nlfam);
             }
-            0x5 | 0x6 | 0x12 => {
-                // accept, accept4, getsockname:
+            0x5 | 0x6 | 0x7 | 0xc | 0x12 => {
+                // accept, accept4, getsockname, getpeername:
                 //
                 // accept{,4} are IP blocklist only.
-                // getsockname is informational.
+                // get{peer,sock}name are informational.
+                // recvfrom is informational.
                 Capability::empty()
             }
             0x2 /* bind */ => Capability::CAP_NET_BIND,
-            _ /* connect, send{to,{m,}msg} */ => Capability::CAP_NET_CONNECT,
+            _ /* connect, send{,to,{m,}msg} */ => Capability::CAP_NET_CONNECT,
         };
         drop(sandbox); // release the read-lock before get-fd.
 
@@ -232,6 +237,20 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // getsockname is used for informational purposes only.
                 return handle_getsockname(fd, &request, args);
             }
+            0x7 => {
+                // getpeername is used for informational purposes only.
+                return handle_getpeername(fd, &request, args);
+            }
+            0x9 => {
+                // send: Connection mode socket.
+                // This only happens via socketcall(2) multiplexer.
+                // On 64-bit we do not hook into send(2).
+                return handle_sendto(fd, args, &request, None, restrict_oob);
+            }
+            0xc => {
+                // recvfrom is used for informational purposes only.
+                return handle_recvfrom(fd, args, &request, restrict_oob);
+            }
             0xf => {
                 // getsockopt is used for informational purposes only.
                 return handle_getsockopt(fd, &request, args);
@@ -278,12 +297,8 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
         } // else we have a valid address to check for access.
 
         let sandbox = request.get_sandbox();
-        let (addr, root) = canon_addr(
-            &request,
-            &sandbox,
-            get_addr(&request, addr_remote, addr_len)?,
-            cap,
-        )?;
+        let argaddr = get_addr(&request, addr_remote, addr_len)?;
+        let (addr, root) = canon_addr(&request, &sandbox, &argaddr, cap)?;
         match addr_family(&addr) {
             PF_UNIX | PF_INET | PF_INET6 => {
                 // Check for access.
@@ -318,9 +333,9 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
 
         // Emulate syscall.
         match op {
-            0x2 => handle_bind(fd, &addr, root.as_ref(), &request, allow_safe_bind),
-            0x3 => handle_connect(fd, &addr, &request, allow_safe_bind),
-            0xb => handle_sendto(fd, args, &request, Some(&addr), restrict_oob),
+            0x2 => handle_bind(fd, (addr, argaddr), root, &request, allow_safe_bind),
+            0x3 => handle_connect(fd, (addr, argaddr), &request, allow_safe_bind),
+            0xb => handle_sendto(fd, args, &request, Some(addr), restrict_oob),
             _ => unreachable!(),
         }
     })
@@ -380,7 +395,7 @@ fn get_addr(
 fn canon_addr<'a>(
     request: &UNotifyEventRequest,
     sandbox: &SandboxGuard,
-    addr: SockaddrStorage,
+    addr: &SockaddrStorage,
     cap: Capability,
 ) -> Result<(SockaddrStorage, Option<CanonicalPath<'a>>), Errno> {
     #[expect(clippy::cast_possible_truncation)]
@@ -412,7 +427,7 @@ fn canon_addr<'a>(
         let pid = request.scmpreq.pid();
         let path = safe_canonicalize(pid, None, &path, fsflags, Some(sandbox.deref()))?;
 
-        let cstr = if path.base.is_empty() {
+        let path_bytes = if path.base.is_empty() {
             // SAFETY: We open a FD to the path and then use the
             // proc path /proc/thread-self/fd/$fd in address' path
             // argument to avoid symlink TOCTOU because connect and
@@ -421,24 +436,30 @@ fn canon_addr<'a>(
             let fd = path.dir.as_ref().unwrap();
             let mut pfd = XPathBuf::from("/proc/thread-self/fd");
             pfd.push_fd(fd.as_raw_fd());
-            CString::new(pfd.as_os_str().as_bytes())
+            pfd.append_byte(0);
+            pfd.into_vec()
         } else {
-            // SAFETY: We split the address into directory and basename
-            // regardless of UNIX_PATH_MAX as we are later going to use
-            // the handler thread to mitigate the TOCTOU vector in the
-            // basename of the UNIX socket address. This is only used
-            // for bind() which does not resolve symbolic links in
-            // basename.
-            CString::new(path.base.as_os_str().as_bytes())
-        }
-        .or(Err(Errno::EINVAL))?;
+            // SAFETY:
+            // 1. We split the address into directory and basename
+            //    regardless of UNIX_PATH_MAX as we are later going to use
+            //    the handler thread to mitigate the TOCTOU vector in the
+            //    basename of the UNIX socket address. This is only used
+            //    for bind() which does not resolve symbolic links in
+            //    basename.
+            // 2. We add "./" for easier identification at recvfrom(2)
+            //    boundary which is for informational purposes only.
+            let mut base = XPathBuf::from("./");
+            base.append_bytes(path.base.as_os_str().as_bytes());
+            base.append_byte(0);
+            base.into_vec()
+        };
 
         // Create sockaddr_un struct.
         let mut sockaddr = libc::sockaddr_un {
             sun_family: libc::AF_UNIX as libc::sa_family_t,
             sun_path: [0; UNIX_PATH_MAX],
         };
-        let socklen = cstr.as_bytes_with_nul().len();
+        let socklen = path_bytes.len();
         if socklen > UNIX_PATH_MAX {
             return Err(Errno::ENAMETOOLONG);
         }
@@ -446,7 +467,7 @@ fn canon_addr<'a>(
         // SAFETY: Copy the bytes without overlapping regions.
         unsafe {
             std::ptr::copy_nonoverlapping(
-                cstr.as_ptr(),
+                path_bytes.as_ptr(),
                 sockaddr.sun_path.as_mut_ptr().cast(),
                 socklen,
             )
@@ -475,7 +496,7 @@ fn canon_addr<'a>(
         Ok((addr, Some(path)))
     } else {
         // No need to canonicalize.
-        Ok((addr, None))
+        Ok((*addr, None))
     }
 }
 
@@ -726,7 +747,6 @@ pub(crate) fn sandbox_addr_notsup(sandbox: &SandboxGuard) -> Result<(), Errno> {
     }
 }
 
-#[inline]
 pub(crate) fn to_msgflags(arg: u64) -> Result<MsgFlags, Errno> {
     let flags = arg.try_into().or(Err(Errno::EINVAL))?;
     MsgFlags::from_bits(flags).ok_or(Errno::EINVAL)
